@@ -1,6 +1,6 @@
 /**
  * Orthanc - A Lightweight, RESTful DICOM Store
- * Copyright (C) 2012 Medical Physics Department, CHU of Liege,
+ * Copyright (C) 2012-2013 Medical Physics Department, CHU of Liege,
  * Belgium
  *
  * This program is free software: you can redistribute it and/or
@@ -663,6 +663,16 @@ namespace Orthanc
     result["ID"] = publicId;
     MainDicomTagsToJson(result, id);
 
+    std::string tmp;
+
+    tmp = db_->GetMetadata(id, MetadataType_AnonymizedFrom);
+    if (tmp.size() != 0)
+      result["AnonymizedFrom"] = tmp;
+
+    tmp = db_->GetMetadata(id, MetadataType_ModifiedFrom);
+    if (tmp.size() != 0)
+      result["ModifiedFrom"] = tmp;
+
     return true;
   }
 
@@ -775,7 +785,8 @@ namespace Orthanc
       // the current resource
       if (!done)
       {
-        assert(db_->LookupParent(currentId, currentId));
+        bool ok = db_->LookupParent(currentId, currentId);
+        assert(ok);
       }
     }
 
@@ -962,4 +973,159 @@ namespace Orthanc
       LOG(INFO) << "Patient " << publicId << " has been unprotected";
   }
 
+
+  void ServerIndex::GetChildInstances(std::list<std::string>& result,
+                                      const std::string& publicId)
+  {
+    result.clear();
+
+    boost::mutex::scoped_lock lock(mutex_);
+
+    ResourceType type;
+    int64_t top;
+    if (!db_->LookupResource(publicId, top, type))
+    {
+      throw OrthancException(ErrorCode_UnknownResource);
+    }
+
+    if (type == ResourceType_Instance)
+    {
+      // The resource is already an instance: Do not go down the hierarchy
+      result.push_back(publicId);
+      return;
+    }
+
+    std::stack<int64_t> toExplore;
+    toExplore.push(top);
+
+    std::list<int64_t> tmp;
+
+    while (!toExplore.empty())
+    {
+      // Get the internal ID of the current resource
+      int64_t resource = toExplore.top();
+      toExplore.pop();
+
+      if (db_->GetResourceType(resource) == ResourceType_Instance)
+      {
+        result.push_back(db_->GetPublicId(resource));
+      }
+      else
+      {
+        // Tag all the children of this resource as to be explored
+        db_->GetChildrenInternalId(tmp, resource);
+        for (std::list<int64_t>::const_iterator 
+               it = tmp.begin(); it != tmp.end(); it++)
+        {
+          toExplore.push(*it);
+        }
+      }
+    }
+  }
+
+
+  void ServerIndex::SetMetadata(const std::string& publicId,
+                                MetadataType type,
+                                const std::string& value)
+  {
+    boost::mutex::scoped_lock lock(mutex_);
+
+    ResourceType rtype;
+    int64_t id;
+    if (!db_->LookupResource(publicId, id, rtype))
+    {
+      throw OrthancException(ErrorCode_UnknownResource);
+    }
+
+    db_->SetMetadata(id, type, value);
+  }
+
+  bool ServerIndex::LookupMetadata(std::string& target,
+                                   const std::string& publicId,
+                                   MetadataType type)
+  {
+    boost::mutex::scoped_lock lock(mutex_);
+
+    ResourceType rtype;
+    int64_t id;
+    if (!db_->LookupResource(publicId, id, rtype))
+    {
+      throw OrthancException(ErrorCode_UnknownResource);
+    }
+
+    return db_->LookupMetadata(target, id, type);
+  }
+
+
+  bool ServerIndex::LookupParent(std::string& target,
+                                 const std::string& publicId)
+  {
+    boost::mutex::scoped_lock lock(mutex_);
+
+    ResourceType type;
+    int64_t id;
+    if (!db_->LookupResource(publicId, id, type))
+    {
+      throw OrthancException(ErrorCode_UnknownResource);
+    }
+
+    int64_t parentId;
+    if (db_->LookupParent(parentId, id))
+    {
+      target = db_->GetPublicId(parentId);
+      return true;
+    }
+    else
+    {
+      return false;
+    }
+  }
+
+
+  uint64_t ServerIndex::IncrementGlobalSequence(GlobalProperty sequence)
+  {
+    boost::mutex::scoped_lock lock(mutex_);
+
+    std::auto_ptr<SQLite::Transaction> transaction(db_->StartTransaction());
+
+    transaction->Begin();
+    uint64_t seq = db_->IncrementGlobalSequence(sequence);
+    transaction->Commit();
+
+    return seq;
+  }
+
+
+
+  void ServerIndex::LogChange(ChangeType changeType,
+                              const std::string& publicId)
+  {
+    boost::mutex::scoped_lock lock(mutex_);
+    std::auto_ptr<SQLite::Transaction> transaction(db_->StartTransaction());
+    transaction->Begin();
+
+    int64_t id;
+    ResourceType type;
+    if (!db_->LookupResource(publicId, id, type))
+    {
+      throw OrthancException(ErrorCode_UnknownResource);
+    }
+
+    db_->LogChange(changeType, id, type);
+
+    transaction->Commit();
+  }
+
+
+  void ServerIndex::DeleteChanges()
+  {
+    boost::mutex::scoped_lock lock(mutex_);
+    db_->ClearTable("Changes");
+  }
+
+  void ServerIndex::DeleteExportedResources()
+  {
+    boost::mutex::scoped_lock lock(mutex_);
+    db_->ClearTable("ExportedResources");
+  }
 }
